#include "Precompiled.h"
#include "WTransferDlg.h"
#include "WNomad.h"
#include "NomadSyncDlg.h"
#include "Track.h"
#include "EzMessageBox.h"
#include "Action.h"


BEGIN_EVENT_TABLE(TransferDlg, wxDialog)
   EVT_BUTTON(ID_ABORT, BaseTransferDlg::OnAbort)
   EVT_TIMER (-1, TransferDlg::OnTimer)
END_EVENT_TABLE()

TransferDlg::TransferDlg(ActionTable* pActionTable, Nomad* pNomad)
	: BaseTransferDlg(pActionTable, pNomad), m_nCurrentItem(-1) 
{
}


TransferDlg::~TransferDlg()
{
}

bool TransferDlg::TransferDataToWindow()
{
	m_timer.SetOwner(this);
	m_timer.Start(10, true);	// only once!
	m_bProcessing = false;
	Init();
	// note: there is a bug which means you cannot change the window where you are
	// sending the notification messages
	m_pNomad->SetCallback((long)GetHandle());
	return wxDialog::TransferDataToWindow();
}

void TransferDlg::OnTimer(wxTimerEvent &WXUNUSED(event) )
{
	m_nCurrentItem = -1;
	m_dBytesDone = 0;
	NextFile();
}

void TransferDlg::NextFile()
{
	if (m_bProcessing) return;
	m_bProcessing = true;
	do
	{
		m_nCurrentItem++;
		if (m_nCurrentItem == m_listAction.size())	 
		{
			Close();
			return;
		}
		if (m_listAction[m_nCurrentItem]->IsPending())
		{
			m_plblCurrTrack->SetLabel(
				m_listAction[m_nCurrentItem]->GetDisplayStr());
			m_pActionTable->ScrollToAction(m_listAction[m_nCurrentItem]);
			wxLogMessage(m_listAction[m_nCurrentItem]->GetDisplayStr());
			m_listAction[m_nCurrentItem]->DoAction(m_pNomad);
			if (m_listAction[m_nCurrentItem]->HadError() || 
				m_listAction[m_nCurrentItem]->IsDone()) 
			{
				m_listAction[m_nCurrentItem]->DoPostAction(0);
				m_dBytesDone += m_listAction[m_nCurrentItem]->GetTimeNeeded();
				SetProgress(m_dBytesDone);
				m_pActionTable->ForceRefresh();
			}
			if (m_nCurrentItem < m_listAction.size() && 
				m_listAction[m_nCurrentItem]->HadError())
			{
				int nRet = EzMessageBox("Error", m_listAction[m_nCurrentItem]->GenErrorMsg(), 
					0, "Continue", "Abort");
				if (nRet == 2) 
				{
					Close();
					return;
				}
			}
		}
	} while (!m_bAbort && !m_listAction[m_nCurrentItem]->GetWaitForCompletion());
	if (m_nCurrentItem < m_listAction.size()) 
		m_listAction[m_nCurrentItem]->SetExecuting(false);
	m_bProcessing = false;
//	wxLogDebug("Finish NextFile\n");
}	 // NextFile

void TransferDlg::RequestClose()
{
	Close();
}

long TransferDlg::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam)
{
	if (nMsg == WM_DAPSDK_ADDITEM_PROGRESS || 
		nMsg == WM_DAPSDK_GETITEM_PROGRESS)
	{
		if (m_nCurrentItem >= m_listAction.size())	
		{
//			wxLogDebug("Extra progress!!: %d, %d\n", wParam, lParam);
			return wxDialog::MSWWindowProc(nMsg, wParam, lParam);
		}
		double dCurrentBytes = m_listAction[m_nCurrentItem]->GetTimeNeeded()/100 
			* wParam;
		SetProgress(m_dBytesDone+dCurrentBytes);
	}
	if (nMsg == WM_DAPSDK_ADDITEM_COMPLETE || 
		nMsg == WM_DAPSDK_GETITEM_COMPLETE)
	{
		// we can get extra complete messages, even after we have finished
		if (m_nCurrentItem >= m_listAction.size() || m_bAbort
			|| wParam == DAPSDK_E_CANCELLED)	
		{
//			wxLogDebug("Extra Complete!!: %d, %d", wParam, lParam);
			return wxDialog::MSWWindowProc(nMsg, wParam, lParam);
		}
		wxLogMessage("Complete: %d, %d", wParam, lParam);
		unsigned int nItemID = 0;
		if (wParam != DAPSDK_SUCCESS)
			m_listAction[m_nCurrentItem]->SetErrorStr(m_pNomad->GetErrorStr(wParam));
		else if (nMsg == WM_DAPSDK_ADDITEM_COMPLETE)
			nItemID = lParam;
		m_listAction[m_nCurrentItem]->DoPostAction(nItemID);
		m_dBytesDone += m_listAction[m_nCurrentItem]->GetTimeNeeded();
		SetProgress(m_dBytesDone);
		m_pActionTable->ForceRefresh();
		MSG msg;
		if (PeekMessage(&msg, NULL, WM_DAPSDK_DOWNLOAD_PROGRESS, 
			WM_DAPSDK_GETITEM_COMPLETE, PM_REMOVE)) 
		{
			wxLogMessage("Extra message!");      
		}
		if (wParam != DAPSDK_SUCCESS)
		{
			int nRet = EzMessageBox("Error", m_listAction[m_nCurrentItem]->GenErrorMsg(), 
				0, "Continue", "Abort");
			if (nRet == 2) 
			{
				Close();
				return wxDialog::MSWWindowProc(nMsg, wParam, lParam);
			}
		}
		NextFile();
	}
	if (nMsg == WM_DAPSDK_JUKEBOX_REMOVAL)
	{
		m_listAction[m_nCurrentItem]->SetErrorStr("Nomad disconnected?");
		wxMessageDialog(NULL, m_listAction[m_nCurrentItem]->GenErrorMsg(), 
			"Error");
		Close();
	}
   return wxDialog::MSWWindowProc(nMsg, wParam, lParam);
}


